#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from typing import List, Callable

import team
from team.filters import Filter
from team.types import Message
from .handler import Handler


class DeletedMessagesHandler(Handler):
    """The deleted messages handler class. Used to handle deleted messages coming from any chat
    (private, group, channel). It is intended to be used with :meth:`~team.Client.add_handler`

    For a nicer way to register this handler, have a look at the
    :meth:`~team.Client.on_deleted_messages` decorator.

    Parameters:
        callback (``Callable``):
            Pass a function that will be called when one or more messages have been deleted.
            It takes *(client, messages)* as positional arguments (look at the section below for a detailed description).

        filters (:obj:`Filters`):
            Pass one or more filters to allow only a subset of messages to be passed
            in your callback function.

    Other parameters:
        client (:obj:`~team.Client`):
            The Client itself, useful when you want to call other API methods inside the message handler.

        messages (List of :obj:`~team.types.Message`):
            The deleted messages, as list.
    """

    def __init__(self, callback: Callable, filters: Filter = None):
        super().__init__(callback, filters)

    async def check(self, client: "team.Client", messages: List[Message]):
        # Every message should be checked, if at least one matches the filter True is returned
        # otherwise, or if the list is empty, False is returned
        for message in messages:
            if await super().check(client, message):
                return True
        else:
            return False



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import functools
import inspect
import threading

from team import types
from team.methods import Methods
from team.methods.utilities import idle as idle_module, compose as compose_module


def async_to_sync(obj, name):
    function = getattr(obj, name)
    main_loop = asyncio.get_event_loop()

    def async_to_sync_gen(agen, loop, is_main_thread):
        async def anext(agen):
            try:
                return await agen.__anext__(), False
            except StopAsyncIteration:
                return None, True

        while True:
            if is_main_thread:
                item, done = loop.run_until_complete(anext(agen))
            else:
                item, done = asyncio.run_coroutine_threadsafe(anext(agen), loop).result()

            if done:
                break

            yield item

    @functools.wraps(function)
    def async_to_sync_wrap(*args, **kwargs):
        coroutine = function(*args, **kwargs)

        try:
            loop = asyncio.get_event_loop()
        except RuntimeError:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)

        if threading.current_thread() is threading.main_thread() or not main_loop.is_running():
            if loop.is_running():
                return coroutine
            else:
                if inspect.iscoroutine(coroutine):
                    return loop.run_until_complete(coroutine)

                if inspect.isasyncgen(coroutine):
                    return async_to_sync_gen(coroutine, loop, True)
        else:
            if inspect.iscoroutine(coroutine):
                if loop.is_running():
                    async def coro_wrapper():
                        return await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(coroutine, main_loop))

                    return coro_wrapper()
                else:
                    return asyncio.run_coroutine_threadsafe(coroutine, main_loop).result()

            if inspect.isasyncgen(coroutine):
                if loop.is_running():
                    return coroutine
                else:
                    return async_to_sync_gen(coroutine, main_loop, False)

    setattr(obj, name, async_to_sync_wrap)


def wrap(source):
    for name in dir(source):
        method = getattr(source, name)

        if not name.startswith("_"):
            if inspect.iscoroutinefunction(method) or inspect.isasyncgenfunction(method):
                async_to_sync(source, name)


# Wrap all Client's relevant methods
wrap(Methods)

# Wrap types' bound methods
for class_name in dir(types):
    cls = getattr(types, class_name)

    if inspect.isclass(cls):
        wrap(cls)

# Special case for idle and compose, because they are not inside Methods
async_to_sync(idle_module, "idle")
idle = getattr(idle_module, "idle")

async_to_sync(compose_module, "compose")
compose = getattr(compose_module, "compose")



################################################################
"""
 Mix-Userbot Open Source . Maintained ? Yes Oh No Oh Yes Ngentot
 
 @ CREDIT : NAN-DEV || Kalo Pake Ini Kode Minimal Cantumkan Credits . Gw Boleh Mikir Juga Anjing, Meski Liat Ultroid
"""
################################################################

import re
import asyncio
from .class_log import LOGGER
from .database import ndB
from Mix import nlx
from random import randint
from os import execvp
from sys import executable
import wget
import random
from pyrogram.errors import *
from pyrogram.raw.functions.messages import DeleteHistory



def extract_api_token(text):
    match = re.search(r"Use this token to access the HTTP API:\s*([\w:]+)", text)
    if match:
        return match.group(1)
    else:
        return None


def extract_api_token_2(text):
    match = re.search(r"You can use this token to access HTTP API:\s*([\w:]+)", text)
    if match:
        return match.group(1)
    else:
        return None


async def autobot():
    LOGGER.info("MEMBUAT BOT TELEGRAM UNTUK ANDA DI @BotFather, Mohon Tunggu")
    gw = nlx.me
    name = gw.first_name + " Asisstant"
    if gw.username:
        username = gw.username + "_bot"
    else:
        username = "mix_" + (str(gw.id))[5:] + "_bot"
    bf = "@BotFather"

    await nlx.send_message(bf, "/start")
    await asyncio.sleep(1)
    await nlx.send_message(bf, "/newbot")
    await asyncio.sleep(1)

    async for aa in nlx.search_messages(bf, limit=1):
        isdone = aa.text
        break
    else:
        isdone = None

    if isdone and isdone.startswith("Sorry,"):
        await nlx.send_message(bf, "/token")
        async for msg in nlx.search_messages(
            bf, query="You can use this token to access HTTP API:", limit=1
        ):
            if msg.text:
                donee = msg.text
                token = extract_api_token_2(donee)
                if token:
                    ndB.set_key("BOT_TOKEN", token)
                    LOGGER.info(
                        f"Selesai. Berhasil membuat @{username} untuk digunakan sebagai bot asisten Anda!"
                    )
                    await enable_inline(username)
                    return
                else:
                    LOGGER.error("Token API tidak ditemukan.")
                    import sys

                    sys.exit(1)

    if isdone and isdone.startswith("Alright, a new bot."):
        await nlx.send_message(bf, name)
    else:
        await nlx.send_message(bf, "/token")
        await asyncio.sleep(2)
        await nlx.send_message(bf, f"@{nlx.me.username}_bot")
        async for msg in nlx.search_messages(
            bf, query="You can use this token to access HTTP API:", limit=1
        ):
            if msg.text:
                donee = msg.text
                token = extract_api_token_2(donee)
                if token:
                    ndB.set_key("BOT_TOKEN", token)
                    LOGGER.info(
                        f"Selesai. Berhasil membuat @{username} untuk digunakan sebagai bot asisten Anda!"
                    )
                    await enable_inline(username)
                    return
                else:
                    LOGGER.error("Token API tidak ditemukan.")
                    import sys

                    sys.exit(1)

    await asyncio.sleep(1)
    async for aa in nlx.search_messages(bf, limit=1):
        isdone = aa.text
        break
    else:
        isdone = None
    if isdone.startswith("Good."):
        await nlx.send_message(bf, username)
    await asyncio.sleep(1)
    async for aa in nlx.search_messages(bf, limit=1):
        isdone = aa.text
        break
    else:
        isdone = None
    if isdone.startswith("Sorry,"):
        ran = randint(1, 100)
        username = "mix_" + (str(gw.id))[6:] + str(ran) + "_bot"
        await nlx.send_message(bf, username)
    await asyncio.sleep(3)
    async for aa in nlx.search_messages(
        bf, query="Use this token to access the HTTP API:", limit=1
    ):
        if aa.text:
            donee = aa.text
            token = extract_api_token(donee)
            if token:
                ndB.set_key("BOT_TOKEN", token)
                LOGGER.info(
                    f"Selesai. Berhasil membuat @{username} untuk digunakan sebagai bot asisten Anda!"
                )
                await enable_inline(username)
            else:
                LOGGER.error("Token API tidak ditemukan.")
                import sys

                sys.exit(1)
        else:
            LOGGER.error(
                "Harap Hapus Beberapa bot Telegram Anda di @Botfather atau Setel Var BOT_TOKEN dengan token bot"
            )
            import sys

            sys.exit(1)


async def enable_inline(username):
    gw = nlx.me
    if gw.username:
        user_name = gw.username
    else:
        user_name = gw.mention
    pp = random.choice(
        [
            "https://telegra.ph//file/19b336da463a05d7d8f8c.jpg",
            "https://telegra.ph//file/2eaf853d09c319465a8f4.jpg",
            "https://telegra.ph//file/7d2e8f0ae636e2f6dc381.jpg",
        ]
    )
    bb = wget.download(pp)
    LOGGER.info(f"Menyesuaikan Bot Asisten di @BotFather")
    bf = "BotFather"
    await nlx.send_message(bf, "/setuserpic")
    await asyncio.sleep(2)
    await nlx.send_message(bf, f"@{username}")
    await asyncio.sleep(2)
    await nlx.send_photo(bf, bb)
    await asyncio.sleep(2)
    await nlx.send_message(bf, "/setabouttext")
    await asyncio.sleep(2)
    await nlx.send_message(bf, f"@{username}")
    await asyncio.sleep(2)
    await nlx.send_message(bf, f"Mix-Userbot Asisten My Owner : @{user_name}")
    await asyncio.sleep(2)
    await nlx.send_message(bf, "/setdescription")
    await asyncio.sleep(2)
    await nlx.send_message(bf, f"@{username}")
    await asyncio.sleep(2)
    await nlx.send_message(
        bf,
        f"Powerful Mix-Userbot Assistant\nMy Owner : @{user_name}\n\nPowered By ~ @KynanSupport",
    )
    await asyncio.sleep(2)
    await nlx.send_message(bf, "/setinline")
    await asyncio.sleep(2)
    await nlx.send_message(bf, f"@{username}")
    await asyncio.sleep(2)
    await nlx.send_message(bf, "Search")
    LOGGER.info("Customisation Done")
    await asyncio.sleep(2)
    execvp(executable, [executable, "-m", "Mix"])


#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import logging
from typing import Union

import team
from team import raw

log = logging.getLogger(__name__)


class GetChatHistoryCount:
    async def get_chat_history_count(
        self: "team.Client",
        chat_id: Union[int, str]
    ) -> int:
        """Get the total count of messages in a chat.

        .. note::

            Due to Telegram latest internal changes, the server can't reliably find anymore the total count of messages
            a **private** or a **basic group** chat has with a single method call. To overcome this limitation, Pyrogram
            has to iterate over all the messages. Channels and supergroups are not affected by this limitation.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.

        Returns:
            ``int``: On success, the chat history count is returned.

        Example:
            .. code-block:: python

                await app.get_history_count(chat_id)
        """

        r = await self.invoke(
            raw.functions.messages.GetHistory(
                peer=await self.resolve_peer(chat_id),
                offset_id=0,
                offset_date=0,
                add_offset=0,
                limit=1,
                max_id=0,
                min_id=0,
                hash=0
            )
        )

        if isinstance(r, raw.types.messages.Messages):
            return len(r.messages)
        else:
            return r.count



################################################################
"""
 Mix-Userbot Open Source . Maintained ? Yes Oh No Oh Yes Ngentot
 
 @ CREDIT : NAN-DEV || Kalo Mo Pake Ini Kode Jangan Hapus Credits Ya Nooobbb
"""
################################################################

from .class_log import LOGGER
from .database import udB
import asyncio
from Mix import nlx


class Emojik:
    def __init__(self):
        self.uprem = nlx.me.is_premium

    def initialize(self):
        if self.uprem == True:
            # ping
            self.ping_var = udB.get_var(nlx.me.id, "emo_ping") or "5269563867305879894"
            self.emo_pinge = int(self.ping_var)
            # pong
            self.pong_var = udB.get_var(nlx.me.id, "emo_pong") or "6183961455436498818"
            self.emo_ponge = int(self.pong_var)
            # proses
            self.proses_var = (
                udB.get_var(nlx.me.id, "emo_proses") or "5974326532670230199"
            )
            self.emo_prosese = int(self.proses_var)
            # sukses
            self.sukses_var = (
                udB.get_var(nlx.me.id, "emo_sukses") or "5021905410089550576"
            )
            self.emo_suksese = int(self.sukses_var)
            # gagal
            self.gagal_var = (
                udB.get_var(nlx.me.id, "emo_gagal") or "5019523782004441717"
            )
            self.emo_gagale = int(self.gagal_var)
            # profil
            self.profil_var = (
                udB.get_var(nlx.me.id, "emo_profil") or "5373012449597335010"
            )
            self.emo_profile = int(self.profil_var)
            # alive
            self.alive_var = (
                udB.get_var(nlx.me.id, "emo_alive") or "4934091419288601395"
            )
            self.emo_alivee = int(self.alive_var)
            # warn
            self.warn_var = udB.get_var(nlx.me.id, "emo_warn") or "6172475875368373616"
            self.emo_warne = int(self.warn_var)
            # block
            self.block_var = (
                udB.get_var(nlx.me.id, "emo_block") or "5240241223632954241"
            )
            self.emo_blocke = int(self.block_var)

        elif self.uprem == False:
            self.ping_var = udB.get_var(nlx.me.id, "emo_ping") or "🏓"
            self.emo_ping = self.ping_var
            # pong
            self.pong_var = udB.get_var(nlx.me.id, "emo_pong") or "🎈"
            self.emo_pong = self.pong_var
            # proses
            self.proses_var = udB.get_var(nlx.me.id, "emo_proses") or "🔄"
            self.emo_proses = self.proses_var
            # sukses
            self.sukses_var = udB.get_var(nlx.me.id, "emo_sukses") or "✅"
            self.emo_sukses = self.sukses_var
            # gagal
            self.gagal_var = udB.get_var(nlx.me.id, "emo_gagal") or "❌"
            self.emo_gagal = self.gagal_var
            # profil
            self.profil_var = udB.get_var(nlx.me.id, "emo_profil") or "👤"
            self.emo_profil = self.profil_var
            # alive
            self.alive_var = udB.get_var(nlx.me.id, "emo_alive") or "🔥"
            self.emo_alive = self.alive_var
            # warn
            self.warn_var = udB.get_var(nlx.me.id, "emo_warn") or "❗"
            self.emo_warn = self.warn_var
            # block
            self.block_var = udB.get_var(nlx.me.id, "emo_block") or "🚫"
            self.emo_block = self.block_var

    @property
    def ping(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_pinge}>🏓</emoji>"
        elif self.uprem == False:
            return f"{self.emo_ping}"

    @property
    def pong(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_ponge}>🥵</emoji>"
        elif self.uprem == False:
            return f"{self.emo_pong}"

    @property
    def proses(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_prosese}>🔄</emoji>"
        elif self.uprem == False:
            return f"{self.emo_proses}"

    @property
    def sukses(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_suksese}>✅</emoji>"
        elif self.uprem == False:
            return f"{self.emo_sukses}"

    @property
    def gagal(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_gagale}>❌</emoji>"
        elif self.uprem == False:
            return f"{self.emo_gagal}"

    @property
    def profil(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_profile}>👤</emoji>"
        elif self.uprem == False:
            return f"{self.emo_profil}"

    @property
    def alive(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_alivee}>⭐</emoji>"
        elif self.uprem == False:
            return f"{self.emo_alive}"

    @property
    def warn(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_warne}>❗️</emoji>"
        elif self.uprem == False:
            return f"{self.emo_warn}"

    @property
    def block(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_blocke}>🚫</emoji>"
        elif self.uprem == False:
            return f"{self.emo_block}"


"""
class Emojik:
    def __init__(self):
        self.uprem = nlx.me.is_premium
    
    def initialize(self):
        if self.uprem == True:
            #ping
            self.ping_var = (
                udB.get_var(nlx.me.id, "emo_ping") or "5269563867305879894"
            )
            self.emo_ping = int(self.ping_var)
            #pong
            self.pong_var = (
                udB.get_var(nlx.me.id, "emo_pong") or "6183961455436498818"
            )
            self.emo_pong = int(self.pong_var)
            #proses
            self.proses_var = (
                udB.get_var(nlx.me.id, "emo_proses") or "5974326532670230199"
            )
            self.emo_proses = int(self.proses_var)
            #sukses
            self.sukses_var = (
                udB.get_var(nlx.me.id, "emo_sukses") or "5021905410089550576"
            )
            self.emo_sukses = int(self.sukses_var)
            #gagal
            self.gagal_var = (
                udB.get_var(nlx.me.id, "emo_gagal") or "5019523782004441717"
            )
            self.emo_gagal = int(self.gagal_var)
            #profil
            self.profil_var = (
                udB.get_var(nlx.me.id, "emo_profil") or "5373012449597335010"
            )
            self.emo_profil = int(self.profil_var)
            #alive
            self.alive_var = (
                udB.get_var(nlx.me.id, "emo_alive") or "4934091419288601395"
            )
            self.emo_alive = int(self.alive_var)
            #warn
            self.warn_var = (
                udB.get_var(nlx.me.id, "emo_warn") or "6172475875368373616"
            )
            self.emo_warn = int(self.warn_var)
            #block
            self.block_var = (
                udB.get_var(nlx.me.id, "emo_block") or "5240241223632954241"
            )
            self.emo_block = int(self.block_var)
            
        elif self.uprem == False:
            self.ping_var = (
                udB.get_var(nlx.me.id, "emo_ping") or "🏓"
            )
            self.emo_ping = self.ping_var
            #pong
            self.pong_var = (
                udB.get_var(nlx.me.id, "emo_pong") or "🎈"
            )
            self.emo_pong = self.pong_var
            #proses
            self.proses_var = (
                udB.get_var(nlx.me.id, "emo_proses") or "🔄"
            )
            self.emo_proses = self.proses_var
            #sukses
            self.sukses_var = (
                udB.get_var(nlx.me.id, "emo_sukses") or "✅"
            )
            self.emo_sukses = self.sukses_var
            #gagal
            self.gagal_var = (
                udB.get_var(nlx.me.id, "emo_gagal") or "❌"
            )
            self.emo_gagal = self.gagal_var
            #profil
            self.profil_var = (
                udB.get_var(nlx.me.id, "emo_profil") or "👤"
            )
            self.emo_profil = self.profil_var
            #alive
            self.alive_var = (
                udB.get_var(nlx.me.id, "emo_alive") or "🔥"
            )
            self.emo_alive = self.alive_var
            #warn
            self.warn_var = (
                udB.get_var(nlx.me.id, "emo_warn") or "❗"
            )
            self.emo_warn = self.warn_var
            #block
            self.block_var = (
                udB.get_var(nlx.me.id, "emo_block") or "🚫"
            )
            self.emo_block = self.block_var

    @property
    def ping(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_ping}>🏓</emoji>"
        elif self.uprem == False:
            return f"{self.emo_ping}"

    @property
    def pong(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_pong}>🥵</emoji>"
        elif self.uprem == False:
            return f"{self.emo_pong}"

    @property
    def proses(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_proses}>🔄</emoji>"
        elif self.uprem == False:
            return f"{self.emo_proses}"

    @property
    def sukses(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_sukses}>✅</emoji>"
        elif self.uprem == False:
            return f"{self.emo_sukses}"

    @property
    def gagal(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_gagal}>❌</emoji>"
        elif self.uprem == False:
            return f"{self.emo_gagal}"

    @property
    def profil(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_profil}>👤</emoji>"
        elif self.uprem == False:
            return f"{self.emo_profil}"

    @property
    def alive(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_alive}>⭐</emoji>"
        elif self.uprem == False:
            return f"{self.emo_alive}"
            
    @property
    def warn(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_warn}>❗️</emoji>"
        elif self.uprem == False:
            return f"{self.emo_warn}"
            
    @property
    def block(self):
        if self.uprem == True:
            return f"<emoji id={self.emo_block}>🚫</emoji>"
        elif self.uprem == False:
            return f"{self.emo_block}"


def inidia():
    emo = Emojik()
    emo.initialize()
    return emo
    
jadinih = inidia()
ping = jadinih.ping
pong = jadinih.pong
proses = jadinih.proses
sukses = jadinih.sukses
gagal = jadinih.gagal
alive = jadinih.alive
profil = jadinih.profil
warn = jadinih.warn
block = jadinih.block
"""